@@ -234,7 +234,7 @@ assuan_command_parse_fd (assuan_context_t ctx, char *line, assuan_fd_t *rfd)
 	return set_error (ctx, GPG_ERR_ASS_SYNTAX, "number required");
 #ifdef HAVE_W32_SYSTEM
       /* Fixme: For a W32/64bit system we will need to change the cast
-         and the conversion fucntion.  */
+         and the conversion function.  */
       *rfd = (void*)strtoul (line, &endp, 10);
 #else
       *rfd = strtoul (line, &endp, 10);
@@ -259,14 +259,31 @@ static gpg_error_t
 std_handler_input (assuan_context_t ctx, char *line)
 {
   gpg_error_t rc;
-  assuan_fd_t fd;
+  assuan_fd_t fd, oldfd;
 
   rc = assuan_command_parse_fd (ctx, line, &fd);
   if (rc)
     return PROCESS_DONE (ctx, rc);
+
+#ifdef HAVE_W32CE_SYSTEM
+  oldfd = fd;
+  fd = _assuan_w32ce_finish_pipe ((int)fd, 0);
+  if (fd == INVALID_HANDLE_VALUE)
+    return PROCESS_DONE (ctx, set_error (ctx, GPG_ERR_ASS_PARAMETER,
+					 "rvid conversion failed"));
+  TRACE2 (ctx, ASSUAN_LOG_SYSIO, "std_handler_input", ctx,
+	  "turned RVID 0x%x into handle 0x%x", oldfd, fd);
+#endif
+
   if (ctx->input_notify_fnc)
-    rc = ctx->input_notify_fnc (ctx, line);
-  if (! rc)
+    {
+      oldfd = ctx->input_fd;
+      ctx->input_fd = fd;
+      rc = ctx->input_notify_fnc (ctx, line);
+      if (rc)
+        ctx->input_fd = oldfd;
+    }
+  else if (!rc)
     ctx->input_fd = fd;
   return PROCESS_DONE (ctx, rc);
 }
@@ -277,14 +294,31 @@ static gpg_error_t
 std_handler_output (assuan_context_t ctx, char *line)
 {
   gpg_error_t rc;
-  assuan_fd_t fd;
-
+  assuan_fd_t fd, oldfd;
+  
   rc = assuan_command_parse_fd (ctx, line, &fd);
   if (rc)
     return PROCESS_DONE (ctx, rc);
+
+#ifdef HAVE_W32CE_SYSTEM
+  oldfd = fd;
+  fd = _assuan_w32ce_finish_pipe ((int)fd, 1);
+  if (fd == INVALID_HANDLE_VALUE)
+    return PROCESS_DONE (ctx, set_error (ctx, gpg_err_code_from_syserror (),
+					 "rvid conversion failed"));
+  TRACE2 (ctx, ASSUAN_LOG_SYSIO, "std_handler_output", ctx,
+	  "turned RVID 0x%x into handle 0x%x", oldfd, fd);
+#endif
+
   if (ctx->output_notify_fnc)
-    rc = ctx->output_notify_fnc (ctx, line);
-  if (!rc)
+    {
+      oldfd = ctx->output_fd;
+      ctx->output_fd = fd;
+      rc = ctx->output_notify_fnc (ctx, line);
+      if (rc)
+        ctx->output_fd = oldfd;
+    }
+  else if (!rc)
     ctx->output_fd = fd;
   return PROCESS_DONE (ctx, rc);
 }
@@ -811,7 +845,9 @@ assuan_get_active_fds (assuan_context_t ctx, int what,
       if (ctx->outbound.fd != ASSUAN_INVALID_FD)
         fdarray[n++] = ctx->outbound.fd;
       if (ctx->outbound.data.fp)
-#ifdef HAVE_W32_SYSTEM
+#if defined(HAVE_W32CE_SYSTEM)
+        fdarray[n++] = (void*)fileno (ctx->outbound.data.fp);
+#elif defined(HAVE_W32_SYSTEM)
         fdarray[n++] = (void*)_get_osfhandle (fileno (ctx->outbound.data.fp));
 #else
         fdarray[n++] = fileno (ctx->outbound.data.fp);
@@ -864,7 +900,7 @@ assuan_get_data_fp (assuan_context_t ctx)
   ctx->outbound.data.error = 0;
   return ctx->outbound.data.fp;
 #else
-  errno = ENOSYS;
+  gpg_err_set_errno (ENOSYS);
   return NULL;
 #endif
 }
